package com.example.goldbach;

import android.app.Activity;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

public class Goldbach extends Activity {

	private EditText et;
	private TextView tv;
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_goldbach);
        Button b = (Button)this.findViewById(R.id.button);
        et = (EditText) findViewById(R.id.edit_text);
        tv  = (TextView) findViewById(R.id.text_view);
        
        // Click en el boton
        b.setOnClickListener(new OnClickListener(){
        	public void onClick(View arg0) {
        		String numText = et.getText().toString();
        		int num = Integer.parseInt(numText);
        		String rS;
        		long tT = System.currentTimeMillis(); // lectura el tiempo inicial
        		if (num > 72000000)
        			rS = goldRuleL(num);
        		else
        			rS = goldRule(num);
        		tT = System.currentTimeMillis()-tT; // lectura del tiempo final, en milisegundos
        		tv.setText(rS+"\nTiempo = "+tT+" ms"); // escritura de datos por pantalla
            }
        });
        
    }
        
	public static String goldRule(int tam){

		// EL LIO  DE ERATOSTENES !!
		// lista con todos los numeros, en princio todos son primos
		int t2 = tam/2;
		boolean[] isP = new boolean[t2];
	        for (int i = 0; i < t2; i++) {
			isP[i] = true;
		}
		isP[0] = false;
		// quito de la lista los que son multiplos de un primo		
		int total = t2;
		for (int i = 1; (2*i+1)*(2*i+1) < tam; i++) {
			if (isP[i]) {
		                for (int j = i; (2*i+1)*(2*j+1) < tam; j++){
					if (isP[(2*i*j)+i+j]){
						isP[(2*i*j)+i+j] = false;
						total--;
					}
				}
			}
		}
		// Escribimos explicitamente el vector de primos, desde el 2
		int[] prim = new int[total];
		prim[0] = 2;
		int j = 1;
		for (int i = 0; i < t2; i++){
			if (isP[i]){
				prim[j] = 2*i+1;
				j++;
			}
		}	
		// empieza GOLDBACH !!
		// variables auxiliares
		int k;
		int sol = 0;
		int end = total-1;
		// variables para sacar la primera y ultima instancia
		int f1 = 0; // primera solucion, numero menor
		int f2 = 0; // primera solucion, numero mayor
		int l1 = 0; // segunda solucion, numero menor
		int l2 = 0; // segunda solucion, numero mayor
		boolean first = true; 
		// escaneo desde el principio de la lista
		int pi;
		for (int i = 0; prim[i] <= t2; i++){
			k = end;			
			// mientras la suma es positiva avanzo el maximo a la baja...	
			pi = prim[i];		
			while(prim[k]+pi > tam){	
				k--; 
			}
			// ya me he pasado, luego contare como maximo desde aqui !
			if (k < total-1) end = k+1;
			// si es solucion, guardar y contar, guardar y contar...
			if (prim[k]+pi == tam){
				if (!first){
					l1 = pi;
					l2 = prim[k];
				}
				else {
					f1 = pi;
					f2 = prim[k];
					first = false;				
				}
				sol++;
			}
		}
		// :)
		return "Primera solucion :\t"+f1+"\t"+f2+"\nUltima solucion :\t"+l1+"\t"+l2+"\nNumero de Soluciones = "+sol;
	}



	public static String goldRuleL(int tam){

		// EL LIO  DE ERATOSTENES !!
		// lista con todos los numeros impares, en princio todos son primos
		int t2 = tam/2;
		int lim = 28000000; // calculo en 2 partes el lio de eratostenes, para que quepa en memoria 'isP'
		int limN = 2*lim+1;
		boolean[] isP = new boolean[lim];
	        for (int i = 0; i < lim; i++) {
			isP[i] = true;
		}
		isP[0] = false;
		// PRIMERA PARTE, hasta 'lim'
		
		// quito de la lista los que son multiplos de un primo		
		for (int i = 1; (2*i+1)*(2*i+1) < limN; i++) {
			if (isP[i]) {
				for (int j1 = i; (2*i+1)*(2*j1+1) < limN; j1++){
					if (isP[(2*i*j1)+i+j1]){
						isP[(2*i*j1)+i+j1] = false;
					}
				}
			}
		}
		// Escribimos explicitamente el vector de primos, desde el 2
		int[] prim = new int[5861455];
		prim[0] = 2;
		int j = 1;
		for (int i = 0; 2*i+1 < limN; i++){
			if (isP[i]){
				prim[j] = 2*i+1;
				j++;
			}
		}	
		// SEGUNDA PARTE, desde 'lim'+1 hasta tam
		// limpio la lista otra vez
		for (int i = 0; i < lim; i++) {
			isP[i] = true;
		}
		// utilizo ahora la lista de primos que he encontrado antes
		for (int i = 1; i < j; i++){
			int k = (int)Math.floor(limN/prim[i])+1;
			if (k%2 == 0) k++;
			while(prim[i]*k < tam) {
				isP[((int)((prim[i]*k-1)/2))-lim] = false;
				k += 2;
			}
		}
		// pongo los primos que faltan
		for (int i = 1; 2*i+limN < tam; i++){
			if (isP[i]){
				prim[j] = 2*i+limN;
				j++;
			}
		}	
		//System.out.println(System.currentTimeMillis()-c);
		// ERATOSTENES DIXIT
/*		j = 0;
		for (int i = 0; i < t2; i++)
			if (prim[i] != 0) {
				j++;	
				System.out.println(prim[i]+" "+j);
			}
*/
		// empieza GOLDBACH !!
		// variables auxiliares
		int k;
		int sol = 0;
		int end = j-1;
		int endP = end;
		// variables para sacar la primera y ultima instancia
		int f1 = 0; // primera solucion, numero menor
		int f2 = 0; // primera solucion, numero mayor
		int l1 = 0; // segunda solucion, numero menor
		int l2 = 0; // segunda solucion, numero mayor
		boolean first = true; 
		// escaneo desde el principio de la lista
		int pi;
		for (int i = 0; prim[i] <= t2; i++){
			k = end;			
			// mientras la suma es positiva avanzo el maxima a la baja...	
			pi = prim[i];	
//			System.out.println(k+" "+prim[k]);	
			while(prim[k]+pi > tam){	
				k--; 
			}
			// ya me he pasado, luego contare como maximo desde aqui !
			if (k < endP-1) end = k+1;
			// si es solucion, guardar y contar, guardar y contar...
			if (prim[k]+pi == tam){
				if (!first){
					l1 = pi;
					l2 = prim[k];
				}
				else {
					f1 = pi;
					f2 = prim[k];
					first = false;				
				}
				sol++;
			}
		}
		// :)
		return "Primera solucion :\t"+f1+"\t"+f2+"\nUltima solucion :\t"+l1+"\t"+l2+"\nNumero de Soluciones = "+sol;
	}    
    
      
}
